'use strict';

var sprintf = require('sprintf');
var rsocketFlowable = require('rsocket-flowable');
var Deferred = require('Deferred');
var rsocketCore = require('rsocket-core');

/**
                                            * Copyright (c) 2017-present, Facebook, Inc.
                                            * All rights reserved.
                                            *
                                            * This source code is licensed under the BSD-style license found in the
                                            * LICENSE file in the root directory of this source tree. An additional grant
                                            * of patent rights can be found in the PATENTS file in the same directory.
                                            *
                                            * 
                                            */

/**
                                                                * A WebSocket transport client for use in browser environments.
                                                                */
class RSocketWebSocketClient$1 {
  constructor(options, encoders) {
    this._encoders = encoders;
    this._options = options;
  }

  connect() {
    return new rsocketFlowable.Single(subscriber => {
      const socket = new WebSocket(this._options.url);
      socket.binaryType = 'arraybuffer';

      const removeListeners = () => {
        socket.removeEventListener('close', onSocketClosed);
        socket.removeEventListener('error', onSocketClosed);
        socket.removeEventListener('open', onOpen);
      };
      const onSocketClosed = () => {
        removeListeners();
        subscriber.onError(
          new Error(
            sprintf(
              'RSocketWebSocketClient: Failed to open connection to %s.',
              this._options.url
            )
          )
        );
      };
      const onOpen = () => {
        removeListeners();
        subscriber.onComplete(
          new WSDuplexConnection(this._options, socket, this._encoders)
        );
      };

      subscriber.onSubscribe(() => {
        removeListeners();
        socket.close();
      });
      socket.addEventListener('close', onSocketClosed);
      socket.addEventListener('error', onSocketClosed);
      socket.addEventListener('open', onOpen);
    });
  }
}

/**
           * @private
           */
class WSDuplexConnection {
  constructor(options, socket, encoders) {
    this.close = () => {
      if (!this._active) {
        return;
      }
      this._active = false;
      this._close.resolve();
      this._receivers.forEach(subscriber => subscriber.onComplete());
      this._receivers.clear();
      this._senders.forEach(subscription => subscription.cancel());
      this._senders.clear();
      this._socket.removeEventListener('close', this._handleClosed);
      this._socket.removeEventListener('error', this._handleClosed);
      this._socket.removeEventListener('message', this._handleMessage);

      this._socket.close();
    };
    this._handleClosed = () => {
      this._handleError(
        new Error('RSocketWebSocketClient: Socket closed unexpectedly.')
      );
    };
    this._handleError = error => {
      this._receivers.forEach(subscriber => subscriber.onError(error));
      this.close();
    };
    this._handleMessage = message => {
      try {
        const frame = this._readFrame(message);
        this._receivers.forEach(subscriber => subscriber.onNext(frame));
      } catch (error) {
        this._handleError(error);
      }
    };
    this._active = true;
    this._close = new Deferred();
    this._encoders = encoders;
    this._options = options;
    this._receivers = new Set();
    this._senders = new Set();
    this._socket = socket;
    this._socket.addEventListener('close', this._handleClosed);
    this._socket.addEventListener('error', this._handleClosed);
    this._socket.addEventListener('message', this._handleMessage);
  }
  sendOne(frame) {
    this._writeFrame(frame);
  }
  send(frames) {
    let subscription;
    frames.subscribe({
      onComplete: () => {
        subscription && this._senders.delete(subscription);
      },
      onError: error => this._handleError(error),
      onNext: frame => this._writeFrame(frame),
      onSubscribe: _subscription => {
        subscription = _subscription;
        subscription.request(Number.MAX_SAFE_INTEGER);
        this._senders.add(subscription);
      },
    });
  }
  receive() {
    return new rsocketFlowable.Flowable(subject => {
      subject.onSubscribe({
        cancel: () => {
          this._receivers.delete(subject);
        },
        request: () => {
          this._receivers.add(subject);
        },
      });
    });
  }
  onClose() {
    return this._close.getPromise();
  }

  _readFrame(message) {
    const buffer = rsocketCore.toBuffer(message.data);
    const frame = this._options.lengthPrefixedFrames
      ? rsocketCore.deserializeFrameWithLength(buffer, this._encoders)
      : rsocketCore.deserializeFrame(buffer, this._encoders);
    if (__DEV__) {
      if (this._options.debug) {
        console.log(rsocketCore.printFrame(frame));
      }
    }
    return frame;
  }

  _writeFrame(frame) {
    try {
      if (__DEV__) {
        if (this._options.debug) {
          console.log(rsocketCore.printFrame(frame));
        }
      }
      const buffer = this._options.lengthPrefixedFrames
        ? rsocketCore.serializeFrameWithLength(frame, this._encoders)
        : rsocketCore.serializeFrame(frame, this._encoders);
      this._socket.send(buffer);
    } catch (error) {
      this._handleError(error);
    }
  }
}

/**
      * Copyright (c) 2017-present, Facebook, Inc.
      * All rights reserved.
      *
      * This source code is licensed under the BSD-style license found in the
      * LICENSE file in the root directory of this source tree. An additional grant
      * of patent rights can be found in the PATENTS file in the same directory.
      *
      * 
      */

module.exports = RSocketWebSocketClient$1;
